// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "MainBountyClass.h"
#include "SideBountyClass.h"
#include "EndlessVendetta/EndlessVendettaCharacter.h"
#include "EndlessVendetta/InteractableActor.h"
#include "EndlessVendetta/UserWidgets/PC_Display.h"
#include "GameFramework/Actor.h"
#include "BountyDirector.generated.h"

UCLASS()
class ENDLESSVENDETTA_API ABountyDirector : public AInteractableActor
{
	GENERATED_BODY()
	
	AEndlessVendettaCharacter* PlayerChar;

	// Scene Component Used to Attach Bounties to
	UArrowComponent* BountyAttachmentPoint;

	// Ordered Array of Main Bounties
	UPROPERTY(EditDefaultsOnly, Category = "Bounty Director")
	TArray<TSubclassOf<AMainBountyClass>> BountyClassArray;

	UPROPERTY(EditDefaultsOnly, Category = "Bounty Director")
	int FavourCost = 500;

	UPROPERTY(EditDefaultsOnly, Category = "Bounty Director")
	TSubclassOf<UPC_Display> PC_DisplayWidgetClass;

	UUserWidget* PC_DisplayWidget;

	int CurrentBountyIndex = 0;

	UPROPERTY(VisibleAnywhere, Category = "Bounty")
	AMainBountyClass* ActiveBounty;

	UPROPERTY(VisibleAnywhere, Category = "Bounty")
	TArray<ASideBountyClass*> ActiveSideBounties;

	// Checks if completed the bounty, and moves onto the next if so
	//UFUNCTION(BlueprintCallable, Category = "Bounty")
	void UpdateBountyProgression();

	void SpawnBountyAndItsSideBounties();
	
	// Collect reward for current Bounty and prepare for the next
	void FinishActiveBounty();

	void Interact() override;
	
protected:
	int Favours = 20;
	// Called when the game starts or when spawned
	virtual void BeginPlay() override;

	/** Blueprint implemented function called from c++, handles aesthetic changes to bounty display
	based on data from ActiveBC and ActiveSBC */
	UFUNCTION(BlueprintImplementableEvent, Category = "bounty")
	void UpdateBountyDisplay();
	
	UFUNCTION()
	void DestroyActiveSideBounties();

	UFUNCTION()
	void DestroyBountyDirectorPCWidget();

	UFUNCTION()
	void EarnFavours(int FavoursEarned);

	UFUNCTION()
	void BuyCustomBountyAlteration_1();

	UFUNCTION(BlueprintCallable, Category = "Bounty Director")
	void BuyCustomBountyAlteration_2();

	UFUNCTION(BlueprintCallable, Category = "Bounty Director")
	void BuyCustomBountyAlteration_3();
	
	UFUNCTION(BlueprintCallable, Category = "Bounty Director")
	void BuyAmmoDrop();

	UFUNCTION(BlueprintCallable, Category = "Bounty Director")
	void BuyHealthDrop();

	UFUNCTION(BlueprintCallable, Category = "Bounty Director")
	void BuyEnemyRadio();

	UFUNCTION(BlueprintCallable, Category = "Bounty Director")
	void BuyFavours();

public:	
	// Called every frame
	virtual void Tick(float DeltaTime) override;

	// Sets default values for this actor's properties
	ABountyDirector();

	// ------ Getters ------
	FString GetBountyTitle()
	{
		return !IsValid(ActiveBounty) ? FString("") : ActiveBounty->GetBountyTitle();
	}

	UTexture2D* GetBountyIcon()
	{
		return !IsValid(ActiveBounty) ? nullptr : ActiveBounty->GetActiveWaypointIcon();
	}
	
	FString GetBountyDescription()
	{
		return !IsValid(ActiveBounty) ? FString("") : ActiveBounty->GetBountyDesc();
	}

	int GetBountyReward()
	{
		return !IsValid(ActiveBounty) ? 0 : ActiveBounty->GetBountyrewardMoney();
	}
	
};
